@@ -1,7 +1,9 @@
 /* 
- * bdajax
+ * bdajax v1.3
  * 
- * Requires: jquery tools
+ * Requires:
+ * - jQuery 1.6.4
+ * - jQuery Tools 1.2.6
  */
 
 (function($) {
@@ -23,7 +25,8 @@
                         var ajax = $(this);
                         ajax.unbind(events);
                         if (ajax.attr('ajax:action')
-                         || ajax.attr('ajax:event')) {
+                         || ajax.attr('ajax:event')
+                         || ajax.attr('ajax:overlay')) {
                             ajax.bind(events, bdajax._dispatching_handler);
                         }
                     }
@@ -113,20 +116,20 @@
             };
         },
         
-        request: function(config) {
-            if (config.url.indexOf('?') != -1) {
-                var addparams = config.params;
-                config.params = bdajax.parsequery(config.url);
-                config.url = bdajax.parseurl(config.url);
+        request: function(options) {
+            if (options.url.indexOf('?') != -1) {
+                var addparams = options.params;
+                options.params = bdajax.parsequery(options.url);
+                options.url = bdajax.parseurl(options.url);
                 for (var key in addparams) {
-                    config.params[key] = addparams[key];
+                    options.params[key] = addparams[key];
                 }
             } else {
-                if (!config.params) { config.params = {}; }
+                if (!options.params) { options.params = {}; }
             }
-            if (!config.type) { config.type = 'html'; }
-            if (!config.error) {
-                config.error = function(req, status, exception) {
+            if (!options.type) { options.type = 'html'; }
+            if (!options.error) {
+                options.error = function(req, status, exception) {
                     if (parseInt(status, 10) === 403) {
                         window.location.pathname = bdajax.default_403;
                     } else {
@@ -136,46 +139,31 @@
                     }
                 };
             }
-            if (!config.cache) { config.cache = false; }
+            if (!options.cache) { options.cache = false; }
             var wrapped_success = function(data, status, request) {
-                config.success(data, status, request);
+                options.success(data, status, request);
                 bdajax.spinner.hide();
             }
             var wrapped_error = function(request, status, error) {
-                config.error(request,
-                             request.status || status,
-                             request.statusText || error);
+                options.error(request,
+                              request.status || status,
+                              request.statusText || error);
                 bdajax.spinner.hide(true);
             }
             bdajax.spinner.show();
             $.ajax({
-                url: config.url,
-                dataType: config.type,
-                data: config.params,
+                url: options.url,
+                dataType: options.type,
+                data: options.params,
                 success: wrapped_success,
                 error: wrapped_error,
-                cache: config.cache
+                cache: options.cache
             });
         },
         
-        action: function(config) {
-            config.params['bdajax.action'] = config.name;
-            config.params['bdajax.mode'] = config.mode;
-            config.params['bdajax.selector'] = config.selector;
-            bdajax.request({
-                url: bdajax.parseurl(config.url) + '/ajaxaction',
-                type: 'json',
-                params: config.params,
-                success: function(data) {
-                    if (!data) {
-                        bdajax.error('Empty response');
-                        bdajax.spinner.hide();
-                    } else {
-                        bdajax.fiddle(data.payload, data.selector, data.mode);
-                        bdajax.continuation(data.continuation);
-                    }
-                }
-            });
+        action: function(options) {
+            options.success = bdajax._ajax_action_success;
+            bdajax._perform_ajax_action(options);
         },
         
         fiddle: function(payload, selector, mode) {
@@ -253,31 +241,44 @@
         overlay: function(options) {
             var elem = $('#ajax-overlay');
             elem.removeData('overlay');
-            elem.overlay({
-                mask: {
-                    color: '#fff',
-                    loadSpeed: 200
-                },
-                onBeforeLoad: function() {
-                    var target = bdajax.parsetarget(options.target);
-                    bdajax.action({
-                        name: options.action,
-                        selector: '#ajax-overlay-content',
-                        mode: 'inner',
-                        url: target.url,
-                        params: target.params
+            var url, params;
+            if (options.target) {
+                var target = bdajax.parsetarget(options.target);
+                url = target.url;
+                params = target.params;
+            } else {
+                url = options.url;
+                params = options.params;
+            }
+            bdajax._perform_ajax_action({
+                name: options.action,
+                selector: '#ajax-overlay-content',
+                mode: 'inner',
+                url: url,
+                params: params,
+                success: function(data) {
+                    bdajax._ajax_action_success(data);
+                    // overlays are not displayed if no payload is received.
+                    if (!data.payload) {
+                        return;
+                    }
+                    elem.overlay({
+                        mask: {
+                            color: '#fff',
+                            loadSpeed: 200
+                        },
+                        onClose: function() {
+                            var overlay = this.getOverlay();
+                            $('#ajax-overlay-content', overlay).html('');
+                        },
+                        oneInstance: false,
+                        closeOnClick: true,
+                        fixed: false
                     });
-                },
-                onClose: function() {
-                    var overlay = this.getOverlay();
-                    $('#ajax-overlay-content', overlay).html('');
-                },
-                oneInstance: false,
-                closeOnClick: true
+                    var overlay = elem.data('overlay');
+                    overlay.load();
+                }
             });
-            var overlay = elem.data('overlay');
-            overlay.load();
-            return overlay;
         },
         
         message: function(message) {
@@ -301,6 +302,7 @@
                 },
                 oneInstance: false,
                 closeOnClick: false,
+                fixed: false,
                 top:'20%'
             });
             elem.data('overlay').load();
@@ -347,6 +349,7 @@
                 },
                 oneInstance: false,
                 closeOnClick: false,
+                fixed: false,
                 top:'20%'
             });
             elem.data('overlay').load();
@@ -354,6 +357,7 @@
         
         _dispatching_handler: function(event) {
             event.preventDefault();
+            event.stopPropagation();
             var elem = $(this);
             var options = {
                 elem: elem,
@@ -376,6 +380,9 @@
             if (elem.attr('ajax:event')) {
                 bdajax._handle_ajax_event(elem);
             }
+            if (elem.attr('ajax:overlay')) {
+                bdajax._handle_ajax_overlay(elem, event);
+            }
         },
         
         _handle_ajax_event: function(elem) {
@@ -388,6 +395,28 @@
             }
         },
         
+        _ajax_action_success: function(data) {
+            if (!data) {
+                bdajax.error('Empty response');
+                bdajax.spinner.hide();
+            } else {
+                bdajax.fiddle(data.payload, data.selector, data.mode);
+                bdajax.continuation(data.continuation);
+            }
+        },
+        
+        _perform_ajax_action: function(options) {
+            options.params['bdajax.action'] = options.name;
+            options.params['bdajax.mode'] = options.mode;
+            options.params['bdajax.selector'] = options.selector;
+            bdajax.request({
+                url: bdajax.parseurl(options.url) + '/ajaxaction',
+                type: 'json',
+                params: options.params,
+                success: options.success
+            });
+        },
+        
         _handle_ajax_action: function(elem, event) {
             var target;
             if (event.ajaxtarget) {
@@ -395,7 +424,7 @@
             } else {
                 target = bdajax.parsetarget(elem.attr('ajax:target'));
             }
-            actions = bdajax._defs_to_array(elem.attr('ajax:action'));
+            var actions = bdajax._defs_to_array(elem.attr('ajax:action'));
             for (var i = 0; i < actions.length; i++) {
                 var defs = actions[i].split(':');
                 bdajax.action({
@@ -408,6 +437,21 @@
             }
         },
         
+        _handle_ajax_overlay: function(elem, event) {
+            var target;
+            if (event.ajaxtarget) {
+                target = event.ajaxtarget;
+            } else {
+                target = bdajax.parsetarget(elem.attr('ajax:target'));
+            }
+            var actionname = elem.attr('ajax:overlay');
+            bdajax.overlay({
+                action: actionname,
+                url: target.url,
+                params: target.params
+            });
+        },
+        
         _defs_to_array: function(str) {
             // XXX: if space in selector when receiving def str, this will fail
             var arr;
